%%% Generates A matrices and simulates pricing experiments to estimate the
%%% sparsity parameters k and d. Produces a plot of (# of experiments) vs 
%%% (estimate of sparsity parameters).


% the number of products
n = 1000;
% parameters for hard sparsity
k = 2;
b = 10;
% parameters for bounded influence
d = 2;
% treat estimated elasticities below this value as 0
thresh = 0.01;
% standard deviation of noise
sigma = 0.1;

%%% generate parameters for hard sparisty or bounded influence; uncomment
%%% the appropriate line
[~, ~, A, ~, ~] = generate_parameters(n, 2, [b k]);
% [~, ~, A, ~, ~] = generate_parameters(n, 3, d);

% try a range of numbers of experiments
sarray = 10:10:100;
% replicate each s this many times
sreps = 1;

% store estimates of sparsity parameters here
sparse_est = zeros(1,length(sarray));
sparse_est_std = zeros(1,length(sarray));

for i = 1:length(sarray)
    s = sarray(i);
    est_array = zeros(1,sreps);
    
    for r = 1:sreps
        % generate random prices and noise terms
        x = 2*randi([0 1], n, s)-1;
        w = normrnd(0, sigma, n, s);
        % calculate demand
        q = A*x + w;
        % estimate the first row of A using Lasso
        [B, FitInfo] = lasso(x',q(1,:)','CV',5);

        %%% recover either k or d; uncomment the appropriate line
        est_array(r) = length(find(abs(B(:,FitInfo.IndexMinMSE)) > thresh));
        % est_array(r) = sum(abs(B(:,FitInfo.IndexMinMSE)));
    end
    sparse_est(i) = mean(est_array);
    sparse_est_std(i) = std(est_array);
end

figure
plot(sarray, sparse_est)
ylim([0 max(sparse_est)])
xlabel('s: # of experiments')
ylabel('estimate of k')
% ylabel('estimate of d')
% title(['n = ' num2str(n) ', d = ' num2str(d)])
title(['n = ' num2str(n) ', k = ' num2str(k)])
